﻿using System;
using System.Collections.Generic;
using TweetinCore.Enum;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.TwitterToken;

namespace TweetinCore.Interfaces
{
    /// <summary>
    /// Contract defining what a user on twitter can do
    /// </summary>
    public interface IUser : ITwitterObject, IEquatable<IUser>
    {
        IUserDTO UserDTO { get; set; }

        #region Twitter API Fields

        long Id { get; }

        string IdStr { get; }

        string Name { get; }

        string ScreenName { get; }

        string Description { get; }

        ITweetDTO Status { get; }

        string Location { get; }

        bool GeoEnabled { get; }

        string Url { get; }

        string Lang { get; }

        int StatusesCount { get; }

        int FollowersCount { get; }

        int FriendsCount { get; }

        bool Following { get; }

        bool Protected { get; }

        bool Verified { get; }

        bool Notifications { get; }

        string ProfileImageUrl { get; }

        string ProfileImageUrlHttps { get; }

        bool FollowRequestSent { get; }

        bool DefaultProfile { get; }

        bool DefaultProfileImage { get; }

        int FavouritesCount { get; }

        int ListedCount { get; }

        string ProfileSidebarFillColor { get; }

        string ProfileSidebarBorderColor { get; }

        string ProfileBackgroundTitle { get; }

        string ProfileBackgroundColor { get; }

        string ProfileBackgroundImageUrl { get; }

        string ProfileBackgroundImageUrlHttps { get; }

        string ProfileTextColor { get; }

        string ProfileLinkColor { get; }

        bool ProfileUseBackgroundImage { get; }

        bool IsTranslator { get; }

        bool ShowAllInlineMedia { get; }

        bool ContributorsEnabled { get; }

        int? UtcOffset { get; }

        string TimeZone { get; }

        #endregion

        #region Tweetinvi API Fields

        /// <summary>
        /// List of friend Ids
        /// </summary>
        List<long> FriendIds { get; set; }

        /// <summary>
        /// List of friends with their profile information
        /// Requires a query per friend user
        /// </summary>
        List<IUser> Friends { get; }

        /// <summary>
        /// List of follower ids
        /// </summary>
        List<long> FollowerIds { get; set; }

        /// <summary>
        /// List of followers with their profile information
        /// Requires a query per friend user
        /// </summary>
        List<IUser> Followers { get; }

        /// <summary>
        /// List of contributors of the account
        /// </summary>
        List<IUser> Contributors { get; set; }

        /// <summary>
        /// List of the account the user is contributing to
        /// </summary>
        List<IUser> Contributees { get; set; }

        /// <summary>
        /// List of tweets as displayed on the timeline
        /// </summary>
        List<ITweet> Timeline { get; set; }

        /// <summary>
        /// List of tweets that actually are retweets
        /// </summary>
        List<ITweet> Retweets { get; set; }

        /// <summary>
        /// List of retweets from friends
        /// </summary>
        List<ITweet> FriendsRetweets { get; set; }

        /// <summary>
        /// Tweets the user created that have been retweeted by followers
        /// </summary>
        List<ITweet> TweetsRetweetedByFollowers { get; set; }

        #endregion

        // Friends
        List<long> GetFriendIds(int maxFriendsToRetrieve = 5000);
        List<IUser> GetFriends(int maxFriendsToRetrieve = 250);

        // Followers
        List<long> GetFollowerIds(int maxFriendsToRetrieve = 5000);
        List<IUser> GetFollowers(int maxFriendsToRetrieve = 250);

        // Friendship
        IRelationship GetRelationshipWith(IUser user);

        // Timeline
        List<ITweet> GetUserTimeline(int maximumTweet = 20, bool excludeReplies = false);

        // Get Favorites
        List<ITweet> GetFavorites(int maximumTweets = 40);

        #region Images

        /// <summary>
        /// Get the Profile Image for a user without specifying a Token / Possibility to download it
        /// </summary>
        /// <param name="size">Size of the image</param>
        /// <param name="https">Calling Https Url</param>
        /// <param name="folderPath">Define location to store it</param>
        /// <returns>Url to access the image from a browser</returns>
        string DownloadProfileImage(ImageSize size = ImageSize.normal, string folderPath = "", bool https = false);

        /// <summary>
        /// Get the Profile Image for a user / Possibility to download it
        /// </summary>
        /// <param name="token"></param>
        /// <param name="size">Size of the image</param>
        /// <param name="https">Calling https Url</param>
        /// <param name="folderPath">Define location to store it</param>
        /// <returns>Filepath of the image</returns>
        string DownloadProfileImage(IToken token, ImageSize size = ImageSize.normal, string folderPath = "", bool https = false);

        #endregion

        #region Contributors

        /// <summary>
        /// Get the list of contributors to the account of the current user
        /// Update the matching attribute of the current user if the parameter is true
        /// Return the list of contributors
        /// </summary>
        /// <param name="createContributorList">False by default. Indicates if the _contributors attribute needs to be updated with the result</param>
        /// <returns>The list of contributors to the account of the current user</returns>
        List<IUser> GetContributors(bool createContributorList = false);

        /// <summary>
        /// Get the list of accounts the current user is allowed to update
        /// Update the matching attribute of the current user if the parameter is true
        /// Return the list of contributees
        /// </summary>
        /// <param name="createContributeeList">False by default. Indicates if the _contributees attribute needs to be updated with the result</param>
        /// <returns>The list of accounts the current user is allowed to update</returns>
        List<IUser> GetContributees(bool createContributeeList = false);

        #endregion

       
    }
}
