﻿using System.Collections.Generic;

namespace TweetinCore.Interfaces.TwitterToken
{
    /// <summary>
    /// User associated with a Token, this "privileged" user
    /// has access private information like messages, timeline...
    /// </summary>
    public interface ITokenUser : IUser
    {
        #region Tweets

        /// <summary>
        /// Send a very simple Tweet with a simple message
        /// </summary>
        /// <param name="text">Text of the tweet</param>
        /// <returns>If the tweet has been sent returns the Tweet</returns>
        ITweet PublishTweet(string text);

        /// <summary>
        /// Send a very simple Tweet with a simple message
        /// </summary>
        /// <param name="tweet">Tweet to be sent</param>
        /// <returns>Tweet has been successfully sent</returns>
        bool PublishTweet(ITweet tweet);

        #endregion

        // Direct Messages
        /// <summary>
        /// List of Messages received
        /// </summary>
        List<IMessage> LatestDirectMessagesReceived { get; }

        /// <summary>
        /// List of messages sent
        /// </summary>
        List<IMessage> LatestDirectMessagesSent { get; }

        /// <summary>
        /// Get the list of direct messages received by the user
        /// Update the matching attribute of the current user if the parameter is true
        /// Return the list of direct messages received by the user
        /// </summary>
        /// <param name="count">Maximum number of messages retrieved</param>
        /// <returns>Collection of direct messages received by the user</returns>
        List<IMessage> GetLatestMessagesReceived(int count = 40);

        List<IMessage> GetLatestMessagesSent(int maximumMessages = 40);

        // Timeline
        /// <summary>
        /// List of tweets as displayed on the Home timeline
        /// Storing the information so that it is not required 
        /// to fetch the data again
        /// </summary>
        List<ITweet> LatestHomeTimeline { get; set; }

        /// <summary>
        /// Get the latest tweets of the TokenUser Home timeline
        /// </summary>
        List<ITweet> GetHomeTimeline(int count = 40, bool excludeReplies = false);

        /// <summary>
        /// List of tweets as displayed on the Mentions timeline
        /// Storing the information so that it is not required 
        /// to fetch the data again
        /// </summary>
        List<IMention> LatestMentionsTimeline { get; set; }

        /// <summary>
        /// Get the latest tweets of the TokenUser Mentions timeline
        /// </summary>
        /// <param name="count">Number of tweets expected</param>
        /// <returns>Tweets of the Mentions timeline of the connected user</returns>
        List<IMention> GetMentionsTimeline(int count = 40);

        // Relationship
        List<IRelationshipState> GetRelationshipStatesWith(List<IUser> users);
        Dictionary<IUser, IRelationshipState> GetRelationshipStatesAssociatedWith(List<IUser> users);

        // Friends - Followers
        List<IUser> GetUsersRequestingFriendship();
        List<IUser> GetUsersYouRequestedToFollow();
        
        bool FollowUser(IUser user);
        bool UnFollowUser(IUser user);

        bool UpdateRelationshipAuthorizationsWith(IUser user, bool retweetsEnabled, bool deviceNotificationsEnabled);

        #region Friends-Followers

        /// <summary>
        /// Follow a user
        /// </summary>
        /// <param name="user">User to follow</param>
        /// <param name="enableNotifications">Should the TokenUser receive notifications from this User</param>
        /// <returns>Friendship has been successfully created</returns>
        bool Follow(IUser user, bool enableNotifications);

        bool Unfollow(IUser user);

        #endregion

        #region Blocked Users

        /// <summary>
        /// User blocked list with their profile information
        /// </summary>
        List<IUser> BlockedUsers { get; set; }

        /// <summary>
        /// User of user ids blocked
        /// </summary>
        List<long> BlockedUsersIds { get; set; }

        /// <summary>
        /// Retrieve the users blocked by the current user.
        /// Populate the corresponding attributes according to the value of the boolean parameters.
        /// Return the list of users blocked by the current user.
        /// </summary>
        /// <param name="createBlockUsers">True by default. Update the attribute _blocked_users if this parameter is true</param>
        /// <param name="createBlockedUsersIds">True by default. Update the attribute _blocked_users_ids if this parameter is true</param>
        /// <returns>Null if there is no valid token for the current user. Otherwise, The list of users blocked by the current user.</returns>
        List<IUser> GetBlockedUsers(bool createBlockUsers = true, bool createBlockedUsersIds = true);

        /// <summary>
        /// Retrieve the ids of the users blocked by the current user.
        /// Populate the corresponding attribute according to the value of the boolean parameter.
        /// Return the list of ids of the users blocked by the current user.
        /// </summary>
        /// <param name="createBlockedUsersIds">True by default. Update the attribute _blocked_users_ids if this parameter is true</param>
        /// <returns>Null if there is no valid token for the current user. Otherwise, The list of ids of the users blocked by the current user.</returns>
        List<long> GetBlockedUsersIds(bool createBlockedUsersIds = true);

        #endregion

        #region List

        /// <summary>
        /// Provide a List users that are likely to be of interest
        /// </summary>
        List<ISuggestedUserList> SuggestedUserList { get; set; }

        /// <summary>
        /// Retrieve the lists of suggested users associated to the current user from the Twitter API.
        /// Update the corresponding attribute according to the value of the parameter. 
        /// Return the lists of suggested users.
        /// </summary>
        /// <param name="createSuggestedUserList">update the _suggestedUserList if true</param>
        /// <returns>null if the token parameter is null, the lists of suggested users otherwise</returns>
        List<ISuggestedUserList> GetSuggestedUserList(bool createSuggestedUserList = true);

        #endregion

        #region Settings

        void GetSettings();

        #endregion
    }
}
