﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using TweetinCore.Helpers;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using Tweetinvi.DTO;
using Tweetinvi.Model;
using Tweetinvi.TwitterEntities;

namespace Tweetinvi.JsonConverters
{
    /// <summary>
    /// Repository of multiple converters required to initialize a DTO property.
    /// </summary>
    public class JsonPropertyConverterRepository : JsonConverter, IJsonPropertyConverterRepository
    {
        public static readonly Dictionary<Type, JsonConverter> JsonConverters;

        static JsonPropertyConverterRepository()
        {
            JsonConverters = new Dictionary<Type, JsonConverter>();

            IntializeClassicalTypesConvertes();
            InitializeTweetinviObjectsConverters();
            InitializeEntitiesConverters();
        }

        private static void IntializeClassicalTypesConvertes()
        {
            var nullableBoolConverter = new JsonTwitterNullableConverter<bool>();
            var nullableLongConverter = new JsonTwitterNullableConverter<long>();
            var nullableDoubleConverter = new JsonTwitterNullableConverter<double>();

            JsonConverters.Add(typeof(bool), nullableBoolConverter);
            JsonConverters.Add(typeof(long), nullableLongConverter);
            JsonConverters.Add(typeof(double), nullableDoubleConverter);
        }

        private static void InitializeTweetinviObjectsConverters()
        {
            var userConverter = new JsonTwitterObjectConverter<IUserDTO, UserDTO>();
            var tweetConverter = new JsonTwitterObjectConverter<ITweetDTO, TweetDTO>();
            var messageConverter = new JsonTwitterObjectConverter<IMessageDTO, MessageDTO>();
            var oembedTweetConverter = new JsonTwitterObjectConverter<IOEmbedTweetDTO, OEmbedTweetDTO>();
            var relationshipConverter = new JsonTwitterObjectConverter<IRelationshipDTO, RelationshipDTO>();
            var relationshipStateConverter = new JsonTwitterObjectConverter<IRelationshipStateDTO, RelationshipStateDTO>();
            var coordinatesConverter = new JsonTwitterObjectConverter<ICoordinates, Coordinates>();
            var geoConverter = new JsonTwitterObjectConverter<IGeo, Geo>();

            JsonConverters.Add(typeof(IUserDTO), userConverter);
            JsonConverters.Add(typeof(ITweetDTO), tweetConverter);
            JsonConverters.Add(typeof(IMessageDTO), messageConverter);
            JsonConverters.Add(typeof(IOEmbedTweetDTO), oembedTweetConverter);
            JsonConverters.Add(typeof(IRelationshipDTO), relationshipConverter);
            JsonConverters.Add(typeof(IRelationshipStateDTO), relationshipStateConverter);
            JsonConverters.Add(typeof(IGeo), geoConverter);
            JsonConverters.Add(typeof(ICoordinates), coordinatesConverter);
        }

        private static void InitializeEntitiesConverters()
        {
            var hashtagEntityConverter = new JsonTwitterObjectConverter<IHashtagEntity, HashtagEntity>();
            var urlEntityConverter = new JsonTwitterObjectConverter<IHashtagEntity, HashtagEntity>();
            var mediaEntityConverter = new JsonTwitterObjectConverter<IHashtagEntity, HashtagEntity>();
            var mediaEntitySizeConverter = new JsonTwitterObjectConverter<IMediaEntitySize, MediaEntitySize>();
            var entitiesConverter = new JsonTwitterObjectConverter<ITweetEntities, TweetEntities>();

            JsonConverters.Add(typeof(IHashtagEntity), hashtagEntityConverter);
            JsonConverters.Add(typeof(IUrlEntity), urlEntityConverter);
            JsonConverters.Add(typeof(IMediaEntity), mediaEntityConverter);
            JsonConverters.Add(typeof(IMediaEntitySize), mediaEntitySizeConverter);
            JsonConverters.Add(typeof(ITweetEntities), entitiesConverter);
        }

        public JsonConverter GetObjectConverter(object objectToConvert)
        {
            return GetTypeConverter(objectToConvert.GetType());
        }

        public JsonConverter GetTypeConverter(Type objectType)
        {
            return JsonConverters[objectType];
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            return GetTypeConverter(objectType).ReadJson(reader, objectType, existingValue, serializer);
        }

        public override bool CanConvert(Type objectType)
        {
            return JsonConverters.ContainsKey(objectType);
        }

        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            throw new NotImplementedException();
        }
    }
}
