﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using TweetinCore.Extensions;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;
using TweetinCore.Interfaces.TwitterToken;
using Tweetinvi.Helpers;
using Tweetinvi.Properties;
using TweetinCore.Events;

namespace Tweetinvi
{
    /// <summary>
    /// A token user is unique to a Token and provides action that will
    /// be executed from the connected user and that are not available
    /// from another user like (read my messages)
    /// </summary>
    public class TokenUser : User, ITokenUser
    {
        private readonly IMessageController _messageController;
        private readonly IFriendshipController _friendshipController;

        #region Attributes

        protected List<IMessage> _directMessagesReceived;
        protected List<IMessage> _directMessagesSent;
        protected List<IMention> _mentionsTimeline;
        protected List<ITweet> _homeTimeline;
        protected List<ISuggestedUserList> _suggested_user_list;

        // List of users blocked by the current user
        protected List<IUser> _blocked_users;
        // List of ids of the users blocked by the current user
        protected List<long> _blocked_users_ids;

        #endregion

        public TokenUser(
           IUserDTO userDTO,
           ITimelineController timelineController,
           IUserController userController,
           IMessageController messageController,
           IFriendshipFactory friendshipFactory,
           IFriendshipController friendshipController)

            : base(userDTO, timelineController, userController, friendshipFactory)
        {
            _messageController = messageController;
            _friendshipController = friendshipController;
        }

        #region Properties

        public TokenUserSettings Settings { get; set; }

        public bool PublishMessage(IMessage message, IUser receiver)
        {
            throw new NotImplementedException();
        }

        public List<IMessage> LatestDirectMessagesReceived
        {
            get { return _directMessagesReceived; }
            protected set { _directMessagesReceived = value; }
        }

        public List<IMessage> LatestDirectMessagesSent
        {
            get { return _directMessagesSent; }
            protected set { _directMessagesSent = value; }
        }

        public List<IMention> LatestMentionsTimeline
        {
            get { return _mentionsTimeline; }
            set { _mentionsTimeline = value; }
        }

        public List<ITweet> LatestHomeTimeline
        {
            get { return _homeTimeline; }
            set { _homeTimeline = value; }
        }

        public List<ISuggestedUserList> SuggestedUserList
        {
            get { return _suggested_user_list; }
            set { _suggested_user_list = value; }
        }

        public List<IUser> BlockedUsers
        {
            get { return _blocked_users; }
            set { _blocked_users = value; }
        }

        public List<long> BlockedUsersIds
        {
            get { return _blocked_users_ids; }
            set { _blocked_users_ids = value; }
        }

        #endregion

        // Home Timeline
        public List<ITweet> GetHomeTimeline(int count = 40, bool excludeReplies = false)
        {
            return _timelineController.GetHomeTimeline(count, excludeReplies);
        }

        public List<IMention> GetMentionsTimeline(int count = 40)
        {
            return _timelineController.GetMentionsTimeline(count);
        }

        // Frienships
        public List<IRelationshipState> GetRelationshipStatesWith(List<IUser> users)
        {
            return _friendshipFactory.GetRelationshipStatesWith(users);
        }

        public Dictionary<IUser, IRelationshipState> GetRelationshipStatesAssociatedWith(List<IUser> users)
        {
            return _friendshipFactory.GetRelationshipStatesAssociatedWith(users);
        }

        // Friends - Followers
        public List<IUser> GetUsersRequestingFriendship()
        {
            return _friendshipController.GetUsersRequestingFriendship();
        }

        public List<IUser> GetUsersYouRequestedToFollow()
        {
            return _friendshipController.GetUsersYouRequestedToFollow();
        }

        public bool FollowUser(IUser user)
        {
            return _friendshipController.CreateFriendshipWith(user);
        }

        public bool UnFollowUser(IUser user)
        {
            return _friendshipController.DestroyFriendshipWith(user);
        }

        public bool UpdateRelationshipAuthorizationsWith(IUser user, bool retweetsEnabled, bool deviceNotificationsEnabled)
        {
            return _friendshipController.UpdateRelationshipAuthorizationsWith(user, retweetsEnabled, deviceNotificationsEnabled);
        }

        // Direct Messages
        public List<IMessage> GetLatestMessagesReceived(int maximumMessages = 40)
        {
            return _messageController.GetLatestMessagesReceived(maximumMessages);
        }

        public List<IMessage> GetLatestMessagesSent(int maximumMessages = 40)
        {
            return _messageController.GetLatestMessagesSent(maximumMessages);
        }

        #region ITokenUser Members

        #region SendTweet

        public ITweet PublishTweet(string text)
        {
            throw new NotImplementedException();
        }

        public bool PublishTweet(ITweet tweet)
        {
            throw new NotImplementedException();
        }

        #endregion

        #region Friends - Followers

        public bool Follow(IUser user, bool enableNotifications)
        {
            if (user == null)
            {
                return false;
            }

            var query = String.Format(Resources.TokenUser_CreateFriendship, enableNotifications);
            if (UserUtils.AddUserInformationInQuery(ref query, user))
            {
                bool result = true;
                WebExceptionHandlingDelegate wex = delegate(WebException exception)
                {
                    if (exception.GetWebExceptionStatusNumber() == 403)
                    {
                        // 403 is expected when the TokenUser is already following the user
                        result = false;
                    }
                };

                var response = _token.ExecutePOSTQuery(query, null, wex);
                return result && response != null;
            }

            return false;
        }

        public bool Unfollow(IUser user)
        {
            if (user == null)
            {
                return false;
            }

            var query = Resources.TokenUser_DestroyFriendship;
            if (UserUtils.AddUserInformationInQuery(ref query, user))
            {
                bool result = true;
                WebExceptionHandlingDelegate wex = delegate(WebException exception)
                {
                    if (exception.GetWebExceptionStatusNumber() == 403)
                    {
                        // 403 is expected when the TokenUser is already following the user
                        result = false;
                    }
                };

                var response = _token.ExecutePOSTQuery(query, null, wex);

                return result && response.ContainsKey("id_str") &&
                       response.GetProp("id_str") as string == user.IdStr;
            }

            return false;
        }

        #endregion

        #region Get Blocked Users

        /// <summary>
        /// Reinitialize the list of ids of the users blocked by the current user if required by the boolean parameter.
        /// Retrieve information about the users blocked by the current user from the Twitter API.
        /// </summary>
        /// <param name="blockedUsersQuery">Request to send to the Twitter API</param>
        /// <param name="del">delegate that will handle the data sent in the Twitter API's response</param>
        /// <param name="createBlockedUsersIds">True by default. Update the attribute _blocked_users_ids if this parameter is true</param>
        /// <returns>Null if there is no valid token for the current user. Otherwise, The list of ids of the users blocked by the current user.</returns>
        private void executeBlockedUsersQuery(string blockedUsersQuery,
            DynamicResponseDelegate del,
            bool createBlockedUsersIds = true)
        {
            // Reset the list of blocked users' ids if required
            if (createBlockedUsersIds)
            {
                _blocked_users_ids = null;
                _blocked_users_ids = new List<long>();
            }

            WebExceptionHandlingDelegate wexDel = delegate(WebException wex)
            {
                int? wexStatusNumber = ExceptionExtension.GetWebExceptionStatusNumber(wex);
                switch (wexStatusNumber)
                {
                    // Handle the Exception when the user's token is not valid
                    case 401:
                        throw new WebException("Blocked users can only be accessed using the user's token. Any other token is rejected.", wex);
                    default:
                        throw wex;
                }
            };

            _token.ExecuteCursorQuery(blockedUsersQuery, 0, Int32.MaxValue, del, wexDel);
        }

        /// <summary>
        /// Retrieve the users blocked by the current user.
        /// Populate the corresponding attributes according to the value of the boolean parameters.
        /// Return the list of users blocked by the current user.
        /// </summary>
        /// <param name="createBlockUsers">True by default. Update the attribute _blocked_users if this parameter is true</param>
        /// <param name="createBlockedUsersIds">True by default. Update the attribute _blocked_users_ids if this parameter is true</param>
        /// <returns>Null if there is no valid token for the current user. Otherwise, The list of users blocked by the current user.</returns>
        public List<IUser> GetBlockedUsers(bool createBlockUsers = true, bool createBlockedUsersIds = true)
        {
            if (_token == null)
            {
                return null;
            }

            List<IUser> blockedUsers = new List<IUser>();
            DynamicResponseDelegate blockedUsersDel = delegate(Dictionary<string, object> twitterBlockedUsers, long previousCursor, long nextCursor)
            {
                // Get the users from the Twitter API's response and store them in the users list
                foreach (var tbu in (IEnumerable<object>)twitterBlockedUsers["users"])
                {
                    //IUser blockedUser = User.Create(tbu);
                    //if (blockedUser != null)
                    //{
                    //    blockedUser.ObjectToken = _shareTokenWithChild ? this._token : null;
                    //    blockedUsers.Add(blockedUser);
                    //    if (createBlockedUsersIds)
                    //    {
                    //        // update the list of ids of the blocked users if required
                    //        if (blockedUser.Id != null)
                    //        {
                    //            _blocked_users_ids.Add(Int64.Parse(blockedUser.Id.ToString()));
                    //        }
                    //    }
                    //}
                }

                throw new NotImplementedException();
                return ((IEnumerable<object>)twitterBlockedUsers["users"]).Count();
            };

            executeBlockedUsersQuery(Resources.TokenUser_GetBlockedUsers, blockedUsersDel);

            // Update the list of blocked users if required
            if (createBlockUsers)
            {
                _blocked_users = blockedUsers;
            }

            return blockedUsers;
        }

        /// <summary>
        /// Retrieve the ids of the users blocked by the current user.
        /// Populate the corresponding attribute according to the value of the boolean parameter.
        /// Return the list of ids of the users blocked by the current user.
        /// </summary>
        /// <param name="createBlockedUsersIds">True by default. Update the attribute _blocked_users_ids if this parameter is true</param>
        /// <returns>Null if there is no valid token for the current user. Otherwise, The list of ids of the users blocked by the current user.</returns>
        public List<long> GetBlockedUsersIds(bool createBlockedUsersIds = true)
        {
            if (_token == null)
            {
                return null;
            }

            List<long> blockedUsersIds = new List<long>();
            DynamicResponseDelegate blockedUsersDel = delegate(Dictionary<string, object> twitterBlockedUsersIds, long previousCursor, long nextCursor)
            {
                // Get the ids of the blocked users from the Twitter API's response and store them in the result list
                foreach (var id in (IEnumerable<object>)twitterBlockedUsersIds["ids"])
                {
                    blockedUsersIds.Add(Int64.Parse(id.ToString()));
                }

                return ((IEnumerable<object>)twitterBlockedUsersIds["ids"]).Count();
            };

            executeBlockedUsersQuery(Resources.TokenUser_GetBlockedUsersIds, blockedUsersDel, createBlockedUsersIds);

            // Update the attribute if required
            if (createBlockedUsersIds)
            {
                _blocked_users_ids = blockedUsersIds;
            }

            return blockedUsersIds;
        }

        #endregion

        #region Get Suggested List
        /// <summary>
        /// Retrieve the lists of suggested users associated to the current user from the Twitter API.
        /// Update the corresponding attribute according to the value of the parameter. 
        /// Return the lists of suggested users.
        /// </summary>
        /// <param name="createSuggestedUserList">update the _suggestedUserList if true</param>
        /// <returns>null if the token parameter is null, the lists of suggested users otherwise</returns>
        public List<ISuggestedUserList> GetSuggestedUserList(bool createSuggestedUserList = true)
        {
            if (_token == null)
            {
                return null;
            }

            List<ISuggestedUserList> suggUserList = new List<ISuggestedUserList>();

            ObjectResponseDelegate objectDelegate = delegate(Dictionary<string, object> responseObject)
            {
                suggUserList.Add(new SuggestedUserList(
                    (string)responseObject["name"],
                    (string)responseObject["slug"],
                    (int)responseObject["size"]));
            };

            // Retrieve the lists of suggested users from the Twitter API
            _token.ExecuteGETQuery(Resources.TokenUser_GetUserSuggestions, objectDelegate);

            // Update the attribute if requested
            if (createSuggestedUserList)
            {
                _suggested_user_list = suggUserList;
            }

            return suggUserList;
        }
        #endregion

        #endregion

        #region Settings

        /// <summary>
        /// Retrieve the settings of the Token's owner
        /// </summary>
        public void GetSettings()
        {
            ObjectResponseDelegate objectDelegate = delegate(Dictionary<string, object> responseObject)
                {
                    Settings = new TokenUserSettings(responseObject);
                };

            _token.ExecuteGETQuery(Resources.TokenUser_GetAccountSettings, objectDelegate);
        }

        #endregion
    }
}
