﻿using System;
using System.Collections.Generic;
using Microsoft.Practices.Unity;
using TweetinCore.Enum;
using TweetinCore.Events;
using TweetinCore.Extensions;
using TweetinCore.Interfaces;
using System.Net;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;
using TweetinCore.Interfaces.TwitterToken;
using Tweetinvi.Helpers;
using Tweetinvi.Properties;

namespace Tweetinvi
{
    /// <summary>
    /// Provide information and functions that a user can do
    /// </summary>
    public class User : TwitterObject, IUser
    {
        protected IUserDTO _userDTO;
        protected readonly ITimelineController _timelineController;
        protected readonly IUserController _userController;
        protected readonly IFriendshipFactory _friendshipFactory;

        public IUserDTO UserDTO
        {
            get { return _userDTO; }
            set { _userDTO = value; }
        }

        #region Public Attributes

        #region Twitter API Attributes

        // This region represents the information accessible from a Twitter API
        // when querying for a User

        public long Id
        {
            get { return _userDTO.Id; }
        }

        public string IdStr
        {
            get { return _userDTO.IdStr; }
        }

        public string Name
        {
            get { return _userDTO.Name; }
        }

        public string ScreenName
        {
            get { return _userDTO.ScreenName; }
        }

        public string Description
        {
            get { return _userDTO.Description; }
        }

        public ITweetDTO Status
        {
            get { return _userDTO.Status; }
        }

        public string Location
        {
            get { return _userDTO.Location; }
        }

        public bool GeoEnabled
        {
            get { return _userDTO.GeoEnabled; }
        }

        public string Url
        {
            get { return _userDTO.Url; }
        }

        public string Lang
        {
            get { return _userDTO.Lang; }
        }

        public int StatusesCount
        {
            get { return _userDTO.StatusesCount; }
        }

        public int FollowersCount
        {
            get { return _userDTO.FollowersCount; }
        }

        public int FriendsCount
        {
            get { return _userDTO.FriendsCount; }
        }

        public bool Following
        {
            get { return _userDTO.Following; }
        }

        public bool Protected
        {
            get { return _userDTO.Protected; }
        }

        public bool Verified
        {
            get { return _userDTO.Verified; }
        }

        public string ProfileImageUrl
        {
            get { return _userDTO.ProfileImageUrl; }
        }

        public string ProfileImageUrlHttps
        {
            get { return _userDTO.ProfileImageUrlHttps; }
        }

        public bool FollowRequestSent
        {
            get { return _userDTO.FollowRequestSent; }
        }

        public bool DefaultProfile
        {
            get { return _userDTO.DefaultProfile; }
        }

        public bool DefaultProfileImage
        {
            get { return _userDTO.DefaultProfileImage; }
        }

        public int FavouritesCount
        {
            get { return _userDTO.FavouritesCount; }
        }

        public int ListedCount
        {
            get { return _userDTO.ListedCount; }
        }

        public string ProfileSidebarFillColor
        {
            get { return _userDTO.ProfileSidebarFillColor; }
        }

        public string ProfileSidebarBorderColor
        {
            get { return _userDTO.ProfileSidebarBorderColor; }
        }

        public string ProfileBackgroundTitle
        {
            get { return _userDTO.ProfileBackgroundTitle; }
        }

        public string ProfileBackgroundColor
        {
            get { return _userDTO.ProfileBackgroundColor; }
        }

        public string ProfileBackgroundImageUrl
        {
            get { return _userDTO.ProfileBackgroundImageUrl; }
        }

        public string ProfileBackgroundImageUrlHttps
        {
            get { return _userDTO.ProfileBackgroundImageUrlHttps; }
        }

        public string ProfileTextColor
        {
            get { return _userDTO.ProfileTextColor; }
        }

        public string ProfileLinkColor
        {
            get { return _userDTO.ProfileLinkColor; }
        }

        public bool ProfileUseBackgroundImage
        {
            get { return _userDTO.ProfileUseBackgroundImage; }
        }

        public bool IsTranslator
        {
            get { return _userDTO.IsTranslator; }
        }

        public bool ShowAllInlineMedia
        {
            get { return _userDTO.ShowAllInlineMedia; }
        }

        public bool ContributorsEnabled
        {
            get { return _userDTO.ContributorsEnabled; }
        }

        public int? UtcOffset
        {
            get { return _userDTO.UtcOffset; }
        }

        public string TimeZone
        {
            get { return _userDTO.TimeZone; }
        }


        [Obsolete("Twitter's documentation states that this property is deprecated")]
        public bool Notifications
        {
            get { return _userDTO.Notifications; }
        }

        #endregion

        #region Tweetinvi API Attributes

        public List<long> FriendIds { get; set; }
        public List<IUser> Friends { get; set; }
        public List<long> FollowerIds { get; set; }
        public List<IUser> Followers { get; set; }
        public List<IUser> Contributors { get; set; }
        public List<IUser> Contributees { get; set; }
        public List<ITweet> Timeline { get; set; }
        public List<ITweet> Retweets { get; set; }
        public List<ITweet> FriendsRetweets { get; set; }
        public List<ITweet> TweetsRetweetedByFollowers { get; set; }

        #endregion

        #endregion

        #region Constructors

        [InjectionConstructor]
        public User(
            IUserDTO userDTO,
            ITimelineController timelineController,
            IUserController userController,
            IFriendshipFactory friendshipFactory)
        {
            _userDTO = userDTO;
            _timelineController = timelineController;
            _userController = userController;
            _friendshipFactory = friendshipFactory;
        }

        #endregion

        #region Private Methods

        #region Get Contributors

        /// <summary>
        /// Update the exception handler attribute with the 3rd parameter
        /// Get the list of users matching the Twitter request url (contributors or contributees)
        /// </summary>
        /// <param name="token"> Current user token to access the Twitter API</param>
        /// <param name="url">Twitter requets URL</param>
        /// <param name="exceptionHandlerDelegate">Delegate method to handle Twitter request exceptions</param>
        /// <returns>Null if the token parameter is null or if the Twitter request fails. A list of users otherwise (contributors or contributees).</returns>
        private List<IUser> GetContributionObjects(IToken token, String url,
            WebExceptionHandlingDelegate exceptionHandlerDelegate = null)
        {
            token = GetQueryToken(token);

            if (token == null)
            {
                Console.WriteLine("User's token is required");
                return null;
            }

            return ResultGenerator.GetUsers(token, url, null, exceptionHandlerDelegate);
        }

        #endregion

        /// <summary>
        /// Add UserId or ScreenName based on the information currently
        /// stored in the user object
        /// </summary>
        /// <param name="query">Query with the user specified</param>
        protected bool AddUserInformationInQuery(ref string query)
        {
            return UserUtils.AddUserInformationInQuery(ref query, this);
        }

        #endregion

        // Friends
        public List<long> GetFriendIds(int maxFriendsToRetrieve = 5000)
        {
            return _userController.GetFriendIds(_userDTO, maxFriendsToRetrieve);
        }

        public List<IUser> GetFriends(int maxFriendsToRetrieve = 250)
        {
            return _userController.GetFriends(_userDTO, maxFriendsToRetrieve);
        }

        // Followers
        public List<long> GetFollowerIds(int maxFriendsToRetrieve = 5000)
        {
            return _userController.GetFollowerIds(_userDTO, maxFriendsToRetrieve);
        }

        public List<IUser> GetFollowers(int maxFriendsToRetrieve = 250)
        {
            return _userController.GetFollowers(_userDTO, maxFriendsToRetrieve);
        }

        // Relationship
        public IRelationship GetRelationshipWith(IUser targetUser)
        {
            if (targetUser == null)
            {
                return null;
            }

            return _friendshipFactory.GetRelationshipBetween(_userDTO, targetUser.UserDTO);
        }

        // Timeline
        public List<ITweet> GetUserTimeline(int maximumTweet = 20, bool excludeReplies = false)
        {
            return _timelineController.GetUserTimeline(_userDTO);
        }

        // Favorites
        public List<ITweet> GetFavorites(int maximumTweets = 40)
        {
            return _userController.GetFavoriteList(_userDTO, maximumTweets);
        }

        #region Download Profile Image

        /// <summary>
        /// Get the Profile Image for a user / Possibility to download it
        /// </summary>
        /// <param name="size">Size of the image</param>
        /// <param name="https">Use encryted communication</param>
        /// <param name="folderPath">Define location to store it</param>
        /// <returns>File path</returns>
        public string DownloadProfileImage(ImageSize size = ImageSize.normal, string folderPath = "", bool https = false)
        {
            return DownloadProfileImage(_token, size, folderPath, https);
        }

        /// <summary>
        /// Get the Profile Image for a user / Possibility to download it
        /// </summary>
        /// <param name="token">Token used to perform the query</param>
        /// <param name="size">Size of the image</param>
        /// <param name="https">Use encryted communication</param>
        /// <param name="folderPath">Define location to store it</param>
        /// <returns>File path</returns>
        public string DownloadProfileImage(IToken token, ImageSize size = ImageSize.normal, string folderPath = "", bool https = false)
        {
            token = GetQueryToken(token);

            if (token == null)
            {
                return null;
            }

            //if (https && String.IsNullOrEmpty(ProfileImageURLHttps) || String.IsNullOrEmpty(ProfileImageURL))
            //{
            //    return null;
            //}

            //string url = https ? ProfileImageURLHttps : ProfileImageURL;
            string imgName = ScreenName ?? IdStr;
            string sizeFormat = size == ImageSize.original ? "" : String.Format("_{0}", size);
            string filePath = String.Format("{0}{1}{2}.jpg", folderPath, imgName, sizeFormat);

            //url = url.Replace("_normal", sizeFormat);

            // Using WebClient to download the image
            using (WebClient client = new WebClient())
            {
                //client.DownloadFile(url, filePath);
            }

            #region Note
            // Using WebRequest
            // if you want to change from WebClient to WebRequest you can simply use the code behind by extending the class
            // WebRequest request = WebRequest.Create(String.Format(query_user_profile_image, screen_name, size));
            // WebResponse response = request.GetResponse(); 
            #endregion

            return filePath;
        }

        #endregion

        #region Get Contributors
        /// <summary>
        /// Get the list of contributors to the account of the current user
        /// Update the matching attribute of the current user if the parameter is true
        /// Return the list of contributors
        /// </summary>
        /// <param name="createContributorList">False by default. Indicates if the _contributors attribute needs to be updated with the result</param>
        /// <returns>The list of contributors to the account of the current user</returns>
        public List<IUser> GetContributors(bool createContributorList = false)
        {
            // Specific error handler
            // Manage the error 400 thrown when contributors are not enabled by the current user
            WebExceptionHandlingDelegate del = delegate(WebException wex)
            {
                int? wexStatusNumber = ExceptionExtension.GetWebExceptionStatusNumber(wex);
                if (wexStatusNumber != null)
                {
                    switch (wexStatusNumber)
                    {
                        case 400:
                            // Don't need to do anything, the method will return null
                            Console.WriteLine("Contributors are not enabled for this user");
                            break;
                        default:
                            // Other errors are not managed
                            throw wex;
                    }
                }
                else
                {
                    throw wex;
                }
            };

            string query = Resources.User_GetContributors;
            if (!AddUserInformationInQuery(ref query))
            {
                return null;
            }

            List<IUser> result = GetContributionObjects(_token, query, del);
            if (createContributorList)
            {
                //_contributors = result;
            }

            return result;
        }
        #endregion

        #region Get Contributees
        /// <summary>
        /// Get the list of accounts the current user is allowed to update
        /// Update the matching attribute of the current user if the parameter is true
        /// Return the list of contributees
        /// </summary>
        /// <param name="createContributeeList">False by default. Indicates if the _contributees attribute needs to be updated with the result</param>
        /// <returns>The list of accounts the current user is allowed to update</returns>
        public List<IUser> GetContributees(bool createContributeeList = false)
        {
            string query = Resources.User_GetContributees;
            if (!AddUserInformationInQuery(ref query))
            {
                return null;
            }

            List<IUser> result = GetContributionObjects(_token, query);

            // Update the _contributees attribute if needed
            if (createContributeeList)
            {
                //_contributees = result;
            }
            return result;
        }

        #endregion

        public override string ToString()
        {
            return _userDTO != null ? _userDTO.Name : "Undefined";
        }

        #region IEquatable<IUser> Members

        /// <summary>
        /// Compare 2 different members and verify if they are the same
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public bool Equals(IUser other)
        {
            //bool result = _id == other.Id && ScreenName == other.ScreenName;

            return false;
        }

        #endregion
    }
}