﻿using System;
using System.Collections.Generic;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.QueryGenerators;
using TweetinCore.Interfaces.TwitterToken;
using TweetinviControllers.Properties;

namespace TweetinviControllers.Direct_Messages
{
    public interface IMessageQueryExecutor
    {
        // Get messages
        List<IMessageDTO> GetLatestMessagesReceived(int maximumMessages = 40);
        List<IMessageDTO> GetLatestMessagesSent(int maximumMessages = 40);

        // Publish Message
        IMessageDTO PublishMessage(IMessageDTO messageDTO);
        IMessageDTO PublishMessage(string messageText, IUserDTO targetUserDTO);
        IMessageDTO PublishMessage(string messageText, string targetUserScreenName);
        IMessageDTO PublishMessage(string messageText, long targetUserId);

        // Detroy Message
        bool DestroyMessage(IMessageDTO messageDTO);
        bool DestroyMessage(long messageId);
    }

    public class MessageQueryExecutor : IMessageQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly IUserQueryGenerator _userQueryGenerator;

        public MessageQueryExecutor(
            ITwitterAccessor twitterAccessor,
            IUserQueryGenerator userQueryGenerator)
        {
            _twitterAccessor = twitterAccessor;
            _userQueryGenerator = userQueryGenerator;
        }

        public List<IMessageDTO> GetLatestMessagesReceived(int maximumMessages = 40)
        {
            string query = String.Format(Resources.Message_GetMessagesReceived, maximumMessages);
            return _twitterAccessor.ExecuteGETQuery<List<IMessageDTO>>(query);
        }

        public List<IMessageDTO> GetLatestMessagesSent(int maximumMessages = 40)
        {
            string query = String.Format(Resources.Messages_GetMessagesSent, maximumMessages);
            return _twitterAccessor.ExecuteGETQuery<List<IMessageDTO>>(query);
        }

        // Publish Message
        public IMessageDTO PublishMessage(IMessageDTO messageDTO)
        {
            if (!CanMessageDTOBePublished(messageDTO))
            {
                return null;
            }

            if (messageDTO.Recipient != null)
            {
                return PublishMessage(messageDTO.Text, messageDTO.Recipient);
            }

            if (messageDTO.RecipientId != -1)
            {
                return PublishMessage(messageDTO.Text, messageDTO.RecipientId);
            }

            return PublishMessage(messageDTO.Text, messageDTO.RecipientScreenName);
        }

        public IMessageDTO PublishMessage(string messageText, IUserDTO targetUserDTO)
        {
            if (!targetUserDTO.HasAValidId())
            {
                return null;
            }

            if (targetUserDTO.Id != -1)
            {
                return PublishMessage(messageText, targetUserDTO.Id);
            }

            return PublishMessage(messageText, targetUserDTO.ScreenName);
        }

        public IMessageDTO PublishMessage(string messageText, string targetUserScreenName)
        {
            string userScreenNameParameter = _userQueryGenerator.GenerateScreenNameParameter(targetUserScreenName);
            string query = String.Format(Resources.Message_NewMessage, messageText, userScreenNameParameter);
            return _twitterAccessor.ExecutePOSTQuery<IMessageDTO>(query);
        }

        public IMessageDTO PublishMessage(string messageText, long targetUserId)
        {
            string userIdParameter = _userQueryGenerator.GenerateUserIdParameter(targetUserId);
            string query = String.Format(Resources.Message_NewMessage, messageText, userIdParameter);
            return _twitterAccessor.ExecutePOSTQuery<IMessageDTO>(query);
        }

        private bool CanMessageDTOBePublished(IMessageDTO messageDTO)
        {
            bool isMessageInValidState = messageDTO != null &&
                                         !messageDTO.IsMessagePublished &&
                                         !messageDTO.IsMessageDestroyed;

            if (!isMessageInValidState)
            {
                return false;
            }

            bool isRecipientValid = messageDTO.Recipient != null ||
                                    messageDTO.RecipientId != -1 ||
                                    !String.IsNullOrEmpty(messageDTO.RecipientScreenName);

            bool messageTextIsValid = !String.IsNullOrEmpty(messageDTO.Text);

            return isRecipientValid && messageTextIsValid;
        }

        // Destroy Message
        public bool DestroyMessage(IMessageDTO messageDTO)
        {
            if (messageDTO == null || messageDTO.Id == -1)
            {
                return false;
            }

            return DestroyMessage(messageDTO.Id);
        }

        public bool DestroyMessage(long messageId)
        {
            string query = String.Format(Resources.Message_DestroyMessage, messageId);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }
    }
}
