﻿using System;
using System.Collections.Generic;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.QueryGenerators;
using TweetinCore.Interfaces.TwitterToken;
using TweetinCore.Interfaces.TwitterToken.QueryDTO;
using TweetinviControllers.Properties;

namespace TweetinviControllers.Friendship
{
    public interface IFriendshipQueryExecutor
    {
        List<long> GetUserIdsRequestingFriendship();
        List<long> GetUserIdsYouRequestedToFollow();

        // Create Friendship
        bool CreateFriendshipWith(IUserIdDTO userDTO);
        bool CreateFriendshipWith(long userId);
        bool CreateFriendshipWith(string userScreenName);

        // Destroy Friendship
        bool DestroyFriendshipWith(IUserIdDTO userDTO);
        bool DestroyFriendshipWith(long userId);
        bool DestroyFriendshipWith(string userScreenName);

        // Update Friendship Authorization
        bool UpdateRelationshipAuthorizationsWith(IUserIdDTO userDTO, bool retweetsEnabled, bool deviceNotifictionEnabled);
        bool UpdateRelationshipAuthorizationsWith(long userId, bool retweetsEnabled, bool deviceNotifictionEnabled);
        bool UpdateRelationshipAuthorizationsWith(string userScreenName, bool retweetsEnabled, bool deviceNotifictionEnabled);
    }

    public class FriendshipQueryExecutor : IFriendshipQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly IUserQueryGenerator _userQueryGenerator;

        public FriendshipQueryExecutor(
            ITwitterAccessor twitterAccessor,
            IUserQueryGenerator userQueryGenerator)
        {
            _twitterAccessor = twitterAccessor;
            _userQueryGenerator = userQueryGenerator;
        }

        public List<long> GetUserIdsRequestingFriendship()
        {
            string query = Resources.Friendship_GetIncomingIds;
            var userIdsDTO = _twitterAccessor.ExecuteCursorGETQuery<IIdsCursorQueryDTO>(query);

            if (userIdsDTO == null)
            {
                return null;
            }

            var userdIds = new List<long>();
            for (int i = 0; i < userIdsDTO.Count; ++i)
            {
                userdIds.AddRange(userIdsDTO[i].Ids);
            }

            return userdIds;
        }

        public List<long> GetUserIdsYouRequestedToFollow()
        {
            string query = Resources.Friendship_GetOutgoingIds;
            var userIdsDTO = _twitterAccessor.ExecuteCursorGETQuery<IIdsCursorQueryDTO>(query);

            if (userIdsDTO == null)
            {
                return null;
            }

            var userdIds = new List<long>();
            for (int i = 0; i < userIdsDTO.Count; ++i)
            {
                userdIds.AddRange(userIdsDTO[i].Ids);
            }

            return userdIds;
        }

        // Create Friendship
        public bool CreateFriendshipWith(IUserIdDTO userDTO)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return false;
            }

            if (userDTO.Id != -1)
            {
                return CreateFriendshipWith(userDTO.Id);
            }

            return CreateFriendshipWith(userDTO.ScreenName);
        }

        public bool CreateFriendshipWith(long userId)
        {
            string userIdParameter = _userQueryGenerator.GenerateUserIdParameter(userId);
            string query = String.Format(Resources.Friendship_Create, userIdParameter);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }

        public bool CreateFriendshipWith(string userScreenName)
        {
            string userScreenNameParameter = _userQueryGenerator.GenerateScreenNameParameter(userScreenName);
            string query = String.Format(Resources.Friendship_Create, userScreenNameParameter);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }

        // Destroy Friendship
        public bool DestroyFriendshipWith(IUserIdDTO userDTO)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return false;
            }

            if (userDTO.Id != -1)
            {
                return DestroyFriendshipWith(userDTO.Id);
            }

            return DestroyFriendshipWith(userDTO.ScreenName);
        }

        public bool DestroyFriendshipWith(long userId)
        {
            string userIdParameter = _userQueryGenerator.GenerateUserIdParameter(userId);
            string query = String.Format(Resources.Friendship_Create, userIdParameter);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }

        public bool DestroyFriendshipWith(string userScreenName)
        {
            string userScreenNameParameter = _userQueryGenerator.GenerateScreenNameParameter(userScreenName);
            string query = String.Format(Resources.Friendship_Create, userScreenNameParameter);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }

        // Update Friendship Authorizations
        public bool UpdateRelationshipAuthorizationsWith(IUserIdDTO userDTO, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return false;
            }

            if (userDTO.Id != -1)
            {
                return UpdateRelationshipAuthorizationsWith(userDTO.Id, retweetsEnabled, deviceNotifictionEnabled);
            }

            return UpdateRelationshipAuthorizationsWith(userDTO.ScreenName, retweetsEnabled, deviceNotifictionEnabled);
        }

        public bool UpdateRelationshipAuthorizationsWith(long userId, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            string userIdParameter = _userQueryGenerator.GenerateUserIdParameter(userId);
            string query = String.Format(Resources.Friendship_Update, retweetsEnabled, deviceNotifictionEnabled, userIdParameter);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }

        public bool UpdateRelationshipAuthorizationsWith(string userScreenName, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            string userScreenNameParameter = _userQueryGenerator.GenerateScreenNameParameter(userScreenName);
            string query = String.Format(Resources.Friendship_Update, retweetsEnabled, deviceNotifictionEnabled, userScreenNameParameter);
            return _twitterAccessor.TryExecutePOSTQuery(query);
        }
    }
}
