﻿using System;
using System.Collections.Generic;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.QueryGenerators;
using TweetinCore.Interfaces.TwitterToken;
using TweetinviControllers.Properties;

namespace TweetinviControllers.Timeline
{
    public interface ITimelineQueryExecutor
    {
        // Home Timeline
        List<ITweetDTO> GetHomeTimeline(int maximumTweets = 40, bool excludeReplies = false);

        // User Timeline
        List<ITweetDTO> GetUserTimeline(IUser user, int maximumTweets = 40, bool excludeReplies = false);
        List<ITweetDTO> GetUserTimeline(IUserIdDTO userDTO, int maximumTweets = 40, bool excludeReplies = false);
        List<ITweetDTO> GetUserTimeline(long userId, int maximumTweets = 40, bool excludeReplies = false);
        List<ITweetDTO> GetUserTimeline(string userScreenName, int maximumTweets = 40, bool excludeReplies = false);

        // Mention Timeline
        List<ITweetDTO> GetMentionsTimeline(int maximumTweets = 40, bool excludeReplies = false);
    }

    public class TimelineQueryExecutor : ITimelineQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly IUserQueryGenerator _userQueryGenerator;

        public TimelineQueryExecutor(
            ITwitterAccessor twitterAccessor,
            IUserQueryGenerator userQueryGenerator)
        {
            _twitterAccessor = twitterAccessor;
            _userQueryGenerator = userQueryGenerator;
        }

        // Home Timeline
        public List<ITweetDTO> GetHomeTimeline(int maximumTweets = 40, bool excludeReplies = false)
        {
            string query = GetHomeTimelineBaseQuery(maximumTweets, excludeReplies);
            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        private string GetHomeTimelineBaseQuery(int maximumTweets, bool excludeReplies)
        {
            return String.Format(Resources.Timeline_GetHomeTimeline, maximumTweets, excludeReplies);
        }

        // User Timeline
        public List<ITweetDTO> GetUserTimeline(IUser user, int maximumTweets = 40, bool excludeReplies = false)
        {
            if (user == null)
            {
                return null;
            }

            return GetUserTimeline(user.UserDTO, maximumTweets, excludeReplies);
        }

        public List<ITweetDTO> GetUserTimeline(IUserIdDTO userDTO, int maximumTweets = 40, bool excludeReplies = false)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return null;
            }

            if (userDTO.Id != -1)
            {
                return GetUserTimeline(userDTO.Id, maximumTweets, excludeReplies);
            }

            return GetUserTimeline(userDTO.ScreenName, maximumTweets, excludeReplies);
        }

        public List<ITweetDTO> GetUserTimeline(long userId, int maximumTweets = 40, bool excludeReplies = false)
        {
            string userIdParameter = _userQueryGenerator.GenerateUserIdParameter(userId);
            string query = GetUserTimelineBaseQuery(userIdParameter, maximumTweets, excludeReplies);

            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        public List<ITweetDTO> GetUserTimeline(string userScreenName, int maximumTweets = 40, bool excludeReplies = false)
        {
            string userScreenNameParameter = _userQueryGenerator.GenerateScreenNameParameter(userScreenName);
            string query = GetUserTimelineBaseQuery(userScreenNameParameter, maximumTweets, excludeReplies);

            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        private string GetUserTimelineBaseQuery(string queryParameter, int maximumTweets, bool excludeReplies)
        {
            string baseQuery = String.Format(Resources.Timeline_GetUserTimeline, maximumTweets, excludeReplies);
            return String.Format("{0}&{1}", baseQuery, queryParameter);
        }

        // Mention Timeline
        public List<ITweetDTO> GetMentionsTimeline(int maximumTweets = 40, bool excludeReplies = false)
        {
            string query = GetMentionsTimelineBaseQuery(maximumTweets, excludeReplies);
            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        private string GetMentionsTimelineBaseQuery(int maximumTweets, bool excludeReplies)
        {
            return String.Format(Resources.Timeline_GetMentionsTimeline, maximumTweets, excludeReplies);
        }
    }
}
