﻿using System.Collections.Generic;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviControllers.Tweet
{
    public class TweetController : ITweetController
    {
        private readonly ITweetQueryExecutor _tweetQueryExecutor;
        private readonly ITweetFactory _tweetFactory;

        public TweetController(
            ITweetQueryExecutor tweetQueryExecutor,
            ITweetFactory tweetFactory)
        {
            _tweetQueryExecutor = tweetQueryExecutor;
            _tweetFactory = tweetFactory;
        }

        // Publish Tweet
        public ITweet PublishTweet(ITweet tweet)
        {
            if (tweet == null)
            {
                return null;
            }

            return PublishTweet(tweet.TweetDTO);
        }

        public ITweet PublishTweet(ITweetDTO tweetDTO)
        {
            var twitterTweetDTO = _tweetQueryExecutor.PublishTweet(tweetDTO);
            return _tweetFactory.GenerateTweetFromDTO(twitterTweetDTO);
        }

        // Publish Tweet InReplyTo
        public ITweet PublishTweetInReplyTo(ITweet tweetToPublish, ITweet tweetToReplyTo)
        {
            if (tweetToPublish == null || tweetToReplyTo == null)
            {
                return null;
            }

            return PublishTweetInReplyTo(tweetToPublish.TweetDTO, tweetToReplyTo.TweetDTO);
        }

        public ITweet PublishTweetInReplyTo(ITweetDTO tweetToPublish, ITweetDTO tweetToReplyTo)
        {
            if (tweetToPublish == null || !IsTweetDTOCurrentlyPublished(tweetToReplyTo))
            {
                return null;
            }

            return PublishTweetInReplyTo(tweetToPublish, tweetToReplyTo.Id);
        }

        public ITweet PublishTweetInReplyTo(ITweetDTO tweet, long tweetIdToRespondTo)
        {
            if (tweet == null)
            {
                return null;
            }

            var twitterTweetDTO = _tweetQueryExecutor.PublishTweetInReplyTo(tweet, tweetIdToRespondTo);
            return _tweetFactory.GenerateTweetFromDTO(twitterTweetDTO);
        }

        // Publish Retweet
        public ITweet PublishRetweet(ITweet tweet)
        {
            if (tweet == null)
            {
                return null;
            }

            return PublishRetweet(tweet.TweetDTO);
        }

        public ITweet PublishRetweet(ITweetDTO tweetDTO)
        {
            if (!IsTweetDTOCurrentlyPublished(tweetDTO))
            {
                return null;
            }

            return PublishRetweet(tweetDTO.Id);
        }

        public ITweet PublishRetweet(long tweetId)
        {
            var twitterTweetDTO = _tweetQueryExecutor.PublishRetweet(tweetId);
            return _tweetFactory.GenerateTweetFromDTO(twitterTweetDTO);
        }

        // Publish GetRetweets
        public List<ITweet> GetRetweets(ITweet tweet)
        {
            if (tweet == null)
            {
                return null;
            }

            return GetRetweets(tweet.TweetDTO);
        }

        public List<ITweet> GetRetweets(ITweetDTO tweetDTO)
        {
            if (!IsTweetDTOCurrentlyPublished(tweetDTO))
            {
                return null;
            }

            return GetRetweets(tweetDTO.Id);
        }

        public List<ITweet> GetRetweets(long tweetId)
        {
            var retweetsDTO = _tweetQueryExecutor.GetRetweets(tweetId);
            return _tweetFactory.GenerateTweetsFromDTO(retweetsDTO);
        }

        // Destroy Tweet
        public bool DestroyTweet(ITweet tweet)
        {
            if (tweet == null)
            {
                return false;
            }

            return DestroyTweet(tweet.TweetDTO);
        }

        public bool DestroyTweet(ITweetDTO tweetDTO)
        {
            if (!IsTweetDTOCurrentlyPublished(tweetDTO))
            {
                return false;
            }

            tweetDTO.IsTweetDestroyed = _tweetQueryExecutor.DestroyTweet(tweetDTO.Id);
            return tweetDTO.IsTweetDestroyed;
        }

        public bool DestroyTweet(long tweetId)
        {
            return _tweetQueryExecutor.DestroyTweet(tweetId);
        }

        // Generate OembedTweet
        public IOEmbedTweet GenerateOEmbedTweet(ITweet tweet)
        {
            if (tweet == null)
            {
                return null;
            }

            return GenerateOEmbedTweet(tweet.TweetDTO);
        }

        public IOEmbedTweet GenerateOEmbedTweet(ITweetDTO tweetDTO)
        {
            var oembedTweetDTO = _tweetQueryExecutor.GenerateOEmbedTweet(tweetDTO);
            return _tweetFactory.GenerateOEmbedTweetDTO(oembedTweetDTO);
        }

        public IOEmbedTweet GenerateOEmbedTweet(long tweetId)
        {
            var oembedTweetDTO = _tweetQueryExecutor.GenerateOEmbedTweet(tweetId);
            return _tweetFactory.GenerateOEmbedTweetDTO(oembedTweetDTO);
        }

        // Favorite Tweet
        public bool FavoriteTweet(ITweet tweet, bool isFavorite)
        {
            if (tweet == null)
            {
                return false;
            }

            return FavoriteTweet(tweet.TweetDTO, isFavorite);
        }

        public bool FavoriteTweet(ITweetDTO tweetDTO, bool isFavorite)
        {
            if (!IsTweetDTOCurrentlyPublished(tweetDTO))
            {
                return false;
            }

            return FavoriteTweet(tweetDTO.Id, isFavorite);
        }

        public bool FavoriteTweet(long tweetId, bool isFavorite)
        {
            return _tweetQueryExecutor.FavoriteTweet(tweetId, isFavorite);
        }

        // Helpers 
        private bool IsTweetDTOCurrentlyPublished(ITweetDTO tweetDTO)
        {
            return tweetDTO != null &&
                   tweetDTO.Id != -1 &&
                   tweetDTO.IsTweetPublished &&
                   !tweetDTO.IsTweetDestroyed;
        }
    }
}
