﻿using System;
using System.Collections.Generic;
using System.Text;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.TwitterToken;
using Tweetinvi.DTO;
using TweetinviControllers.Properties;

namespace TweetinviControllers.Tweet
{
    public interface ITweetQueryExecutor
    {
        // Publish Tweet
        ITweetDTO PublishTweet(ITweetDTO tweet);
        ITweetDTO PublishTweetInReplyTo(ITweetDTO tweet, long tweetIdToRespondTo);

        // Publish Retweet
        ITweetDTO PublishRetweet(long tweetId);

        // Get Retweets
        List<ITweetDTO> GetRetweets(long tweetId);

        // Destroy Tweet
        bool DestroyTweet(long tweetId);

        // Generate OEmbedTweet
        IOEmbedTweetDTO GenerateOEmbedTweet(ITweetDTO tweetDTO);
        IOEmbedTweetDTO GenerateOEmbedTweet(long tweetId);

        // Favorite Tweet
        bool FavoriteTweet(long tweetId, bool isFavorite);
    }

    public class TweetQueryExecutor : ITweetQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly ITweetQueryGenerator _tweetQueryGenerator;

        public TweetQueryExecutor(
            ITwitterAccessor twitterAccessor,
            ITweetQueryGenerator tweetQueryGenerator)
        {
            _twitterAccessor = twitterAccessor;
            _tweetQueryGenerator = tweetQueryGenerator;
        }

        private bool CanTweetDTOBePublished(ITweetDTO tweetDTO)
        {
            return tweetDTO != null;
        }

        private bool IsTweetDTOPublished(ITweetDTO tweetDTO)
        {
            return tweetDTO != null && tweetDTO.IsTweetPublished && !tweetDTO.IsTweetDestroyed;
        }

        private string AddAdditionalParameters(ITweetDTO tweet, string baseQuery)
        {
            StringBuilder query = new StringBuilder(baseQuery);

            query.Append(_tweetQueryGenerator.GeneratePlaceIdParameter(tweet.PlaceId));
            query.Append(_tweetQueryGenerator.GenerateGeoParameter(tweet.Coordinates));

            return query.ToString();
        }

        // Publish Tweet
        public ITweetDTO PublishTweet(ITweetDTO tweetDTO)
        {
            if (!CanTweetDTOBePublished(tweetDTO))
            {
                return null;
            }

            string baseQuery = String.Format(Resources.Tweet_Publish, tweetDTO.Text);
            string query = AddAdditionalParameters(tweetDTO, baseQuery);

            return _twitterAccessor.ExecutePOSTQuery<TweetDTO>(query);
        }

        public ITweetDTO PublishTweetInReplyTo(ITweetDTO tweetDTO, long tweetIdToRespondTo)
        {
            if (!CanTweetDTOBePublished(tweetDTO))
            {
                return null;
            }

            string baseQuery = String.Format(Resources.Tweet_PublishInReplyTo, tweetDTO.Text, tweetIdToRespondTo);
            string query = AddAdditionalParameters(tweetDTO, baseQuery);

            return _twitterAccessor.ExecutePOSTQuery<TweetDTO>(query);
        }

        // Publish Retweet
        public ITweetDTO PublishRetweet(long tweetId)
        {
            string query = String.Format(Resources.Tweet_Retweet_Publish, tweetId);
            return _twitterAccessor.ExecutePOSTQuery<TweetDTO>(query);
        }

        // Get Retweets
        public List<ITweetDTO> GetRetweets(long tweetId)
        {
            string query = String.Format(Resources.Tweet_Retweet_GetRetweets, tweetId);
            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        // Generate OEmbed Tweet
        public IOEmbedTweetDTO GenerateOEmbedTweet(ITweetDTO tweetDTO)
        {
            if (!IsTweetDTOPublished(tweetDTO))
            {
                return null;
            }

            return GenerateOEmbedTweet(tweetDTO.Id);
        }

        public IOEmbedTweetDTO GenerateOEmbedTweet(long tweetId)
        {
            string query = String.Format(Resources.Tweet_GenerateOEmbed, tweetId);
            return _twitterAccessor.ExecuteGETQuery<IOEmbedTweetDTO>(query);
        }

        // Destroy Tweet
        public bool DestroyTweet(long tweetId)
        {
            string query = String.Format(Resources.Tweet_Destroy, tweetId);

            TweetDTO tweetDTO;
            return _twitterAccessor.TryExecutePOSTQuery(query, out tweetDTO);
        }

        // Favourite Tweet
        public bool FavoriteTweet(long tweetId, bool isFavorite)
        {
            string query = String.Format(Resources.Tweet_Favorite_Create, tweetId);

            if (isFavorite == false)
            {
                query = String.Format(Resources.Tweet_Favorite_Destroy, tweetId);
            }

            return _twitterAccessor.TryExecutePOSTQuery(query);
        }
    }
}