﻿using System.Collections.Generic;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviControllers.User
{
    /// <summary>
    /// Reason for change : Twitter changes the operation exposed on its REST API
    /// </summary>
    public class UserController : IUserController
    {
        private readonly IUserQueryExecutor _userQueryExecutor;
        private readonly ITweetFactory _tweetFactory;
        private readonly IUserFactory _userFactory;
        private readonly ITimelineController _timelineController;

        public UserController(
            IUserQueryExecutor userQueryExecutor,
            ITweetFactory tweetFactory,
            IUserFactory userFactory,
            ITimelineController timelineController)
        {
            _userQueryExecutor = userQueryExecutor;
            _tweetFactory = tweetFactory;
            _userFactory = userFactory;
            _timelineController = timelineController;
        }

        // Timeline
        public List<ITweet> GetHomeTimeline(IUser user)
        {
            if (user == null)
            {
                return null;
            }

            return GetHomeTimeline(user.UserDTO);
        }

        public List<ITweet> GetHomeTimeline(IUserIdDTO userDTO)
        {
            return _timelineController.GetUserTimeline(userDTO);
        }

        // Friend Ids
        public List<long> GetFriendIds(IUser user, int maxFriendsToRetrieve = 5000)
        {
            if (user == null)
            {
                return null;
            }

            return GetFriendIds(user.UserDTO, maxFriendsToRetrieve);
        }

        public List<long> GetFriendIds(IUserIdDTO userDTO, int maxFriendsToRetrieve = 5000)
        {
            return _userQueryExecutor.GetFriendIds(userDTO, maxFriendsToRetrieve);
        }

        // Friends
        public List<IUser> GetFriends(IUser user, int maxFriendsToRetrieve = 250)
        {
            if (user == null)
            {
                return null;
            }

            return GetFriends(user.UserDTO);
        }

        public List<IUser> GetFriends(IUserIdDTO userDTO, int maxFriendsToRetrieve = 250)
        {
            var friendIds = GetFriendIds(userDTO, maxFriendsToRetrieve);
            return _userFactory.GetUsersFromIds(friendIds);
        }

        // Follower Ids
        public List<long> GetFollowerIds(IUser user, int maxFriendsToRetrieve = 5000)
        {
            if (user == null)
            {
                return null;
            }

            return GetFollowerIds(user, maxFriendsToRetrieve);
        }

        public List<long> GetFollowerIds(IUserIdDTO userDTO, int maxFollowersToRetrieve = 5000)
        {
            return _userQueryExecutor.GetFollowersIds(userDTO, maxFollowersToRetrieve);
        }

        // Followers
        public List<IUser> GetFollowers(IUser user, int maxFollowersToRetrieve = 250)
        {
            if (user == null)
            {
                return null;
            }

            return GetFollowers(user.UserDTO);
        }

        public List<IUser> GetFollowers(IUserIdDTO userDTO, int maxFollowersToRetrieve = 250)
        {
            var followerIds = GetFollowerIds(userDTO, maxFollowersToRetrieve);
            return _userFactory.GetUsersFromIds(followerIds);
        }

        // Favorites
        public List<ITweet> GetFavoriteList(IUser user, int maxFavoritesToRetrieve = 40)
        {
            if (user == null)
            {
                return null;
            }

            return GetFavoriteList(user.UserDTO, maxFavoritesToRetrieve);
        }

        public List<ITweet> GetFavoriteList(IUserIdDTO userDTO, int maxFavoritesToRetrieve = 40)
        {
            var favoriteTweetsDTO = _userQueryExecutor.GetFavoriteList(userDTO, maxFavoritesToRetrieve);
            return _tweetFactory.GenerateTweetsFromDTO(favoriteTweetsDTO);
        }

        public List<ITweet> GetFavoriteList(long userId, int maxFavoritesToRetrieve = 40)
        {
            var favoriteTweetsDTO = _userQueryExecutor.GetFavoriteList(userId, maxFavoritesToRetrieve);
            return _tweetFactory.GenerateTweetsFromDTO(favoriteTweetsDTO);
        }

        public List<ITweet> GetFavoriteList(string userScreenName, int maxFavoritesToRetrieve = 40)
        {
            var favoriteTweetsDTO = _userQueryExecutor.GetFavoriteList(userScreenName, maxFavoritesToRetrieve);
            return _tweetFactory.GenerateTweetsFromDTO(favoriteTweetsDTO);
        }
    }
}
