﻿using System;
using System.Collections.Generic;
using System.Linq;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.QueryGenerators;
using TweetinCore.Interfaces.TwitterToken;
using TweetinCore.Interfaces.TwitterToken.QueryDTO;
using TweetinviControllers.Properties;

namespace TweetinviControllers.User
{
    public interface IUserQueryExecutor
    {
        // Friend Ids
        List<long> GetFriendIds(IUserIdDTO userDTO, int maxFriendsToRetrieve);
        List<long> GetFriendIds(long userId, int maxFriendsToRetrieve);
        List<long> GetFriendIds(string userScreenName, int maxFriendsToRetrieve);

        // Followers Ids
        List<long> GetFollowersIds(IUserIdDTO userDTO, int maxFollowersToRetrieve);
        List<long> GetFollowersIds(long userId, int maxFollowersToRetrieve);
        List<long> GetFollowersIds(string userScreenName, int maxFollowersToRetrieve);

        // Favorites
        List<ITweetDTO> GetFavoriteList(IUserIdDTO userDTO, int maxFavoritesToRetrieve);
        List<ITweetDTO> GetFavoriteList(long userId, int maxFavoritesToRetrieve);
        List<ITweetDTO> GetFavoriteList(string userScreenName, int maxFavoritesToRetrieve);
    }

    public class UserQueryExecutor : IUserQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly IUserQueryGenerator _userQueryGenerator;

        public UserQueryExecutor(
            ITwitterAccessor twitterAccessor,
            IUserQueryGenerator userQueryGenerator)
        {
            _twitterAccessor = twitterAccessor;
            _userQueryGenerator = userQueryGenerator;
        }

        // Friend ids
        public List<long> GetFriendIds(IUserIdDTO userDTO, int maxFriendsToRetrieve)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return null;
            }

            if (userDTO.Id != -1)
            {
                return GetFriendIds(userDTO.Id, maxFriendsToRetrieve);
            }

            return GetFriendIds(userDTO.ScreenName, maxFriendsToRetrieve);
        }

        public List<long> GetFriendIds(long userId, int maxFriendsToRetrieve)
        {
            string query = String.Format(Resources.User_GetFriends_FromUserId, userId);
            return ExecuteGetUserIdsQuery(query, maxFriendsToRetrieve);
        }

        public List<long> GetFriendIds(string userScreenName, int maxFriendsToRetrieve)
        {
            string query = String.Format(Resources.User_GetFriends_FromScreenName, userScreenName);
            return ExecuteGetUserIdsQuery(query, maxFriendsToRetrieve);
        }

        // Followers
        public List<long> GetFollowersIds(IUserIdDTO userDTO, int maxFollowersToRetrieve)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return null;
            }

            if (userDTO.Id != -1)
            {
                return GetFollowersIds(userDTO.Id, maxFollowersToRetrieve);
            }

            return GetFollowersIds(userDTO.ScreenName, maxFollowersToRetrieve);
        }

        public List<long> GetFollowersIds(long userId, int maxFollowersToRetrieve)
        {
            string query = String.Format(Resources.User_GetFollowers_FromUserId, userId);
            return ExecuteGetUserIdsQuery(query, maxFollowersToRetrieve);
        }

        public List<long> GetFollowersIds(string userScreenName, int maxFollowersToRetrieve)
        {
            string query = String.Format(Resources.User_GetFollowers_FromScreenName, userScreenName);
            return ExecuteGetUserIdsQuery(query, maxFollowersToRetrieve);
        }

        // Favorites
        public List<ITweetDTO> GetFavoriteList(IUserIdDTO userDTO, int maxFavoritesToRetrieve)
        {
            if (userDTO == null || !userDTO.HasAValidId())
            {
                return null;
            }

            if (userDTO.Id != -1)
            {
                return GetFavoriteList(userDTO.Id, maxFavoritesToRetrieve);
            }

            return GetFavoriteList(userDTO.ScreenName, maxFavoritesToRetrieve);
        }

        public List<ITweetDTO> GetFavoriteList(long userId, int maxFavoritesToRetrieve)
        {
            string userIdParameter = _userQueryGenerator.GenerateUserIdParameter(userId);
            string query = String.Format(Resources.User_GetFavorites, userIdParameter, maxFavoritesToRetrieve);
            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        public List<ITweetDTO> GetFavoriteList(string userScreenName, int maxFavoritesToRetrieve)
        {
            string userIdParameter = _userQueryGenerator.GenerateScreenNameParameter(userScreenName);
            string query = String.Format(Resources.User_GetFavorites, userIdParameter, maxFavoritesToRetrieve);
            return _twitterAccessor.ExecuteGETQuery<List<ITweetDTO>>(query);
        }

        // Helpers
        private List<long> ExecuteGetUserIdsQuery(string query, int maxUsersToRetrieve)
        {
            var userIdsDTO = _twitterAccessor.ExecuteCursorGETQuery<IIdsCursorQueryDTO>(query, maxUsersToRetrieve);
            if (userIdsDTO == null)
            {
                return null;
            }

            var userdIds = new List<long>();
            for (int i = 0; i < userIdsDTO.Count - 1; ++i)
            {
                userdIds.AddRange(userIdsDTO[i].Ids);
            }

            // TODO : Move the limit logic in the ExecuteCursorQuery
            var userIdsDTOResult = userIdsDTO.Last();
            userdIds.AddRange(userIdsDTOResult.Ids.Take(maxUsersToRetrieve - userdIds.Count));

            return userdIds;
        }
    }
}
