﻿using System;
using System.Collections.Generic;
using System.Linq;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.QueryGenerators;
using TweetinCore.Interfaces.TwitterToken;
using TweetinviFactories.Properties;
using TweetinviFactories.User;

namespace TweetinviFactories.Friendship
{
    public interface IFriendshipFactoryQueryExecutor
    {
        // Get Existing Relationship
        IRelationshipDTO GetRelationshipBetween(IUserDTO sourceUserDTO, IUserDTO targetUserDTO);
        IRelationshipDTO GetRelationshipBetween(long sourceUserId, long targetUserId);
        IRelationshipDTO GetRelationshipBetween(long sourceUserId, string targetUserScreenName);
        IRelationshipDTO GetRelationshipBetween(string sourceUserScreenName, long targetUserId);
        IRelationshipDTO GetRelationshipBetween(string sourceUserScreenName, string targetUserScreenName);

        // Get Multiple Relationships
        List<IRelationshipStateDTO> GetRelationshipStatesWith(List<IUserDTO> targetUsersDTO);
        List<IRelationshipStateDTO> GetRelationshipStatesWith(List<long> targetUsersId);
        List<IRelationshipStateDTO> GetRelationshipStatesWith(List<string> targetUsersScreenName);
    }

    public class FriendshipFactoryQueryExecutor : IFriendshipFactoryQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly IUserQueryGenerator _userQueryGenerator;
        private readonly IUserFactoryQueryGenerator _userFactoryQueryGenerator;

        public FriendshipFactoryQueryExecutor(
            ITwitterAccessor twitterAccessor,
            IUserQueryGenerator userQueryGenerator,
            IUserFactoryQueryGenerator userFactoryQueryGenerator)
        {
            _twitterAccessor = twitterAccessor;
            _userQueryGenerator = userQueryGenerator;
            _userFactoryQueryGenerator = userFactoryQueryGenerator;
        }

        // Get Existing Relationship
        public IRelationshipDTO GetRelationshipBetween(IUserDTO sourceUserDTO, IUserDTO targetUserDTO)
        {
            string sourceParameter = _userQueryGenerator.GenerateIdOrScreenNameParameter(sourceUserDTO, "source_id", "source_screen_name");
            string targetParameter = _userQueryGenerator.GenerateIdOrScreenNameParameter(targetUserDTO, "target_id", "target_screen_name");
            string query = String.Format(Resources.Friendship_GetRelationship, sourceParameter, targetParameter);

            return _twitterAccessor.ExecuteGETQuery<IRelationshipDTO>(query);
        }

        public IRelationshipDTO GetRelationshipBetween(long sourceUserId, long targetUserId)
        {
            string sourceParameter = _userQueryGenerator.GenerateUserIdParameter(sourceUserId);
            string targetParameter = _userQueryGenerator.GenerateUserIdParameter(targetUserId);
            string query = String.Format(Resources.Friendship_GetRelationship, sourceParameter, targetParameter);

            return _twitterAccessor.ExecuteGETQuery<IRelationshipDTO>(query);
        }

        public IRelationshipDTO GetRelationshipBetween(long sourceUserId, string targetUserScreenName)
        {
            string sourceParameter = _userQueryGenerator.GenerateUserIdParameter(sourceUserId);
            string targetParameter = _userQueryGenerator.GenerateScreenNameParameter(targetUserScreenName);
            string query = String.Format(Resources.Friendship_GetRelationship, sourceParameter, targetParameter);

            return _twitterAccessor.ExecuteGETQuery<IRelationshipDTO>(query);
        }

        public IRelationshipDTO GetRelationshipBetween(string sourceUserScreenName, long targetUserId)
        {
            string sourceParameter = _userQueryGenerator.GenerateScreenNameParameter(sourceUserScreenName);
            string targetParameter = _userQueryGenerator.GenerateUserIdParameter(targetUserId);
            string query = String.Format(Resources.Friendship_GetRelationship, sourceParameter, targetParameter);

            return _twitterAccessor.ExecuteGETQuery<IRelationshipDTO>(query);
        }

        public IRelationshipDTO GetRelationshipBetween(string sourceUserScreenName, string targetUserScreenName)
        {
            string sourceParameter = _userQueryGenerator.GenerateScreenNameParameter(sourceUserScreenName);
            string targetParameter = _userQueryGenerator.GenerateScreenNameParameter(targetUserScreenName);
            string query = String.Format(Resources.Friendship_GetRelationship, sourceParameter, targetParameter);

            return _twitterAccessor.ExecuteGETQuery<IRelationshipDTO>(query);
        }

        // Get Relationship with
        public List<IRelationshipStateDTO> GetRelationshipStatesWith(List<IUserDTO> targetUsersDTO)
        {
            string userIdsAndScreenNameParameter = _userFactoryQueryGenerator.GenerateListOfUserDTOParameter(targetUsersDTO);
            string query = String.Format(Resources.Friendship_GetRelationships, userIdsAndScreenNameParameter);

            return _twitterAccessor.ExecuteGETQuery<List<IRelationshipStateDTO>>(query);
        }

        public List<IRelationshipStateDTO> GetRelationshipStatesWith(List<long> targetUsersId)
        {
            string userIds = _userFactoryQueryGenerator.GenerateListOfIdsParameter(targetUsersId);
            string userIdsParameter = String.Format("user_id={0}", userIds);
            string query = String.Format(Resources.Friendship_GetRelationships, userIdsParameter);

            return _twitterAccessor.ExecuteGETQuery<List<IRelationshipStateDTO>>(query);
        }

        public List<IRelationshipStateDTO> GetRelationshipStatesWith(List<string> targetUsersScreenName)
        {
            string userScreenNames = _userFactoryQueryGenerator.GenerateListOfScreenNameParameter(targetUsersScreenName);
            string userScreenNamesParameter = String.Format("screen_name={0}", userScreenNames);
            string query = String.Format(Resources.Friendship_GetRelationships, userScreenNamesParameter);

            return _twitterAccessor.ExecuteGETQuery<List<IRelationshipStateDTO>>(query);
        }
    }
}
