﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.Practices.Unity;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;
using TweetinCore.Interfaces.TwitterToken;

namespace TweetinviFactories.User
{
    public class UserFactory : IUserFactory
    {
        private readonly IUserFactoryQueryExecutor _userFactoryQueryExecutor;
        private readonly IUnityFactory<ITokenUser> _tokenUserUnityFactory;
        private readonly IUnityFactory<IUser> _userUnityFactory;
        private readonly IUnityFactory<IUserIdDTO> _userIdDTOUnityFactory;

        public UserFactory(
            IUserFactoryQueryExecutor userFactoryQueryExecutor,
            IUnityFactory<ITokenUser> tokenUserUnityFactory,
            IUnityFactory<IUser> userUnityFactory,
            IUnityFactory<IUserIdDTO> userIdDTOUnityFactory)
        {
            _userFactoryQueryExecutor = userFactoryQueryExecutor;
            _tokenUserUnityFactory = tokenUserUnityFactory;
            _userUnityFactory = userUnityFactory;
            _userIdDTOUnityFactory = userIdDTOUnityFactory;
        }

        public ITokenUser GetCurrentUser()
        {
            var userDTO = _userFactoryQueryExecutor.GetCurrentUser();
            return GenerateTokenUserFromDTO(userDTO);
        }

        public IUser GetUserFromId(long userId)
        {
            var userDTO = _userFactoryQueryExecutor.GetUserDTOFromId(userId);
            return GenerateUserFromDTO(userDTO);
        }

        public IUser GetUserFromScreenName(string userName)
        {
            var userDTO = _userFactoryQueryExecutor.GetUserDTOFromScreenName(userName);
            return GenerateUserFromDTO(userDTO);
        }

        public List<IUser> GetUsersFromIds(List<long> userIds)
        {
            var usersDTO = _userFactoryQueryExecutor.GetUsersDTOFromIds(userIds);
            return GenerateUsersFromDTO(usersDTO);
        }

        public List<IUser> GetUsersFromNames(List<string> userNames)
        {
            var usersDTO = _userFactoryQueryExecutor.GetUsersDTOFromScreenNames(userNames);
            return GenerateUsersFromDTO(usersDTO);
        }

        // Generate DTO from id
        public IUserIdDTO GenerateUserIdDTOFromId(long userId)
        {
            var userIdDTO = _userIdDTOUnityFactory.Create();
            userIdDTO.Id = userId;

            return userIdDTO;
        }

        public IUserIdDTO GenerateUserIdDTOFromScreenName(string userScreenName)
        {
            var userIdDTO = _userIdDTOUnityFactory.Create();
            userIdDTO.ScreenName = userScreenName;

            return userIdDTO;
        }

        // Generate from DTO
        public ITokenUser GenerateTokenUserFromDTO(IUserDTO userDTO)
        {
            if (userDTO == null)
            {
                return null;
            }

            var user = _tokenUserUnityFactory.Create(new ResolverOverride[]
            {
                new ParameterOverride("userDTO", userDTO)
            });

            return user;
        }

        public IUser GenerateUserFromDTO(IUserDTO userDTO)
        {
            if (userDTO == null)
            {
                return null;
            }

            var user = _userUnityFactory.Create(new ResolverOverride[]
            {
                new ParameterOverride("userDTO", userDTO)
            });

            return user;
        }

        public List<IUser> GenerateUsersFromDTO(IEnumerable<IUserDTO> usersDTO)
        {
            if (usersDTO == null)
            {
                return null;
            }

            return usersDTO.Select(GenerateUserFromDTO).ToList();
        }
    }
}
