﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using TweetinCore.Enum;
using TweetinCore.Interfaces.oAuth;
using TweetinCore.Interfaces.TwitterToken;
using TweetinCore.Interfaces.TwitterToken.QueryDTO;
using TweetinCore.Wrappers;
using Tweetinvi.JsonConverters;
using TwitterToken.QueryJsonConverters;

namespace TwitterToken
{
    public class TwitterAccessor : ITwitterAccessor
    {
        private ITokenCredentials _twitterCredentials;
        
        private readonly IOAuthToken _oAuthToken;
        private readonly IJObjectWrapper _jObjectWrapper;
        private readonly IJsonConvertWrapper _jsonConvertWrapper;

        public TwitterAccessor(
            IOAuthToken oAuthToken,
            IJObjectWrapper jObjectWrapper,
            IJsonConvertWrapper jsonConvertWrapper)
        {
            _oAuthToken = oAuthToken;
            _jObjectWrapper = jObjectWrapper;
            _jsonConvertWrapper = jsonConvertWrapper;
        }

        public ITokenCredentials TwitterCredentials
        {
            get { return _twitterCredentials; }
            set
            {
                _twitterCredentials = value;
                _oAuthToken.Credentials = _twitterCredentials;
            }
        }

        // Execute
        public JObject ExecuteGETQuery(string query)
        {
            string jsonResponse = ExecuteQuery(query, HttpMethod.GET);
            return _jObjectWrapper.GetJobjectFromJson(jsonResponse);
        }

        public JObject ExecutePOSTQuery(string query)
        {
            string jsonResponse = ExecuteQuery(query, HttpMethod.POST);
            return _jObjectWrapper.GetJobjectFromJson(jsonResponse);
        }

        // Execute<T>
        public T ExecuteGETQuery<T>(string query, JsonConverter[] converters = null) where T : class
        {
            string jsonResponse = ExecuteQuery(query, HttpMethod.GET);

            if (converters == null)
            {
                return _jsonConvertWrapper.DeserializeObject<T>(jsonResponse, JsonPropertiesConverterRepository.Converters);
            }

            return _jsonConvertWrapper.DeserializeObject<T>(jsonResponse, converters);
        }

        public T ExecutePOSTQuery<T>(string query, JsonConverter[] converters = null) where T : class
        {
            string jsonResponse = ExecuteQuery(query, HttpMethod.POST);

            if (converters == null)
            {
                return _jsonConvertWrapper.DeserializeObject<T>(jsonResponse, JsonPropertiesConverterRepository.Converters);
            }

            return _jsonConvertWrapper.DeserializeObject<T>(jsonResponse, converters);
        }

        // Try Execute
        public bool TryExecuteGETQuery(string query, JsonConverter[] converters = null)
        {
            try
            {
                ExecuteGETQuery(query);
                return true;
            }
            catch (WebException)
            {
                return false;
            }
        }

        public bool TryExecutePOSTQuery(string query, JsonConverter[] converters = null)
        {
            try
            {
                ExecutePOSTQuery(query);
                return true;
            }
            catch (WebException)
            {
                return false;
            }
        }

        public bool TryExecuteGETQuery<T>(string query, out T resultObject, JsonConverter[] converters = null) where T : class
        {
            try
            {
                resultObject = ExecuteGETQuery<T>(query, converters);
                return true;
            }
            catch (WebException)
            {
                resultObject = null;
                return false;
            }
        }

        public bool TryExecutePOSTQuery<T>(string query, out T resultObject, JsonConverter[] converters = null) where T : class
        {
            try
            {
                resultObject = ExecutePOSTQuery<T>(query, converters);
                return true;
            }
            catch (WebException)
            {
                resultObject = null;
                return false;
            }
        }

        // Cursor Query
        private T ExecuteCursorQuery<T>(string baseQuery,long cursor) where T : class, IBaseCursorQueryDTO
        {
            var query = String.Format("{0}cursor={1}", baseQuery, cursor);

            T dtoResult;
            if (TryExecuteGETQuery(query, out dtoResult, JsonQueryConverterRepository.Converters))
            {
                return dtoResult;
            }

            return null;
        }

        public List<T> ExecuteCursorGETQuery<T>(
                    string baseQuery, 
                    int maxObjectToRetrieve = Int32.MaxValue,
                    long cursor = -1) 
               where T : class, IBaseCursorQueryDTO
        {
            int nbOfObjectsProcessed = 0;
            long previousCursor = -2;
            long nextCursor = cursor;

            // add & for query parameters
            if (baseQuery.Contains("?") && baseQuery.Last() != '?')
            {
                baseQuery += "&";
            }

            var result = new List<T>();
            while (previousCursor != nextCursor && nbOfObjectsProcessed < maxObjectToRetrieve)
            {
                T cursorResult = ExecuteCursorQuery<T>(baseQuery, cursor);

                if (cursorResult == null)
                {
                    return result;
                }

                nbOfObjectsProcessed += cursorResult.GetNumberOfObjectRetrieved();
                previousCursor = cursorResult.PreviousCursor;
                nextCursor = cursorResult.NextCursor;

                result.Add(cursorResult);
            }

            return result;
        }

        // Concrete Execute
        public string ExecuteQuery(string query, HttpMethod method)
        {
            return _oAuthToken.ExecuteQuery(query, method, null);
        }
    }
}
